import sys
import json
import pathlib
from scrub.tools.parsers import translate_results

WARNING_LEVEL = 'Low'
ID_PREFIX = 'pylint'


def parse_warnings(raw_input_file, parsed_output_file):
    """This function parses the raw PyLint warnings into the SCRUB format.

    Inputs:
        - raw_input_file: Absolute path to the raw PyLint output file [string]
        - parsed_output_file: Absolute path to the file where the parsed warnings will be stored [string]
    """

    # Initialize the variables
    warning_count = 1

    # Read in the input data
    with open(raw_input_file, 'r') as input_fh:
        input_data = json.loads(input_fh.read())

    # Iterate through every finding in the input file
    raw_warnings = []
    for finding in input_data:
        # Parse the finding
        warning_file = pathlib.Path(finding['path']).resolve()
        warning_line = int(finding['line'])
        warning_description = finding['message'].splitlines()
        warning_message = ['[Type: ' + finding['message-id'] + ']' + ' [' + finding['type'] + ']'] + warning_description
        warning_id = ID_PREFIX + str(warning_count).zfill(3)
        warning_type = finding['symbol']

        # Add to the warning dictionary
        raw_warnings.append(translate_results.create_warning(warning_id, warning_file, warning_line, warning_message,
                                                             ID_PREFIX, WARNING_LEVEL, warning_type))

        # Increment the warning count
        warning_count = warning_count + 1

    # Create the SCRUB output file
    translate_results.create_scrub_output_file(raw_warnings, parsed_output_file)


if __name__ == '__main__':
    parse_warnings(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]))



import sys
import pathlib
import logging
from scrub.tools.parsers import translate_results

WARNING_LEVEL = 'Low'
ID_PREFIX = 'gcc'


def parse_warnings(raw_input_file, parsed_output_file):
    """This function parses the raw GCC compiler warnings into the SCRUB format.

    Inputs:
        - raw_input_file: Absolute path to the raw GCC compiler log containing warnings [string]
        - parsed_output_file: Absolute path to the file where the parsed warnings will be stored [string]
    """

    # Initialize the variables
    warning_count = 1
    warning_list = []
    raw_warnings = []
    warning_id = None
    warning_file = None
    warning_line = None
    warning_message = []
    parsing = False
    description = False

    # Print a status message
    logging.info('')
    logging.info('\tParsing results...')
    logging.info('\t>> Executing command: get_gcc_warnings.parse_warnings(%s, %s)', str(raw_input_file),
                 str(parsed_output_file))
    logging.info('\t>> From directory: %s', str(pathlib.Path().absolute()))

    # Read in the input data
    with open(raw_input_file, 'r') as input_fh:
        input_data = input_fh.readlines()

    # Iterate through every line of the input file
    for line in input_data:
        # Find lines that contain warnings
        if ('in function' in line.lower() or 'in file' in line.lower()) and not parsing:
            parsing = True

        if parsing and not description and 'warning:' in line.lower():
            description = True

            # Split the line and store the data
            warning_file = pathlib.Path(line.split(':')[0].strip()).resolve()
            warning_line = int(line.split(':')[1].strip())
            warning_message = ['GCC Compiler Warning:', '\t' + line.rstrip()]
            warning_id = ID_PREFIX + str(warning_count).zfill(3)

        elif parsing and description and line.lower().startswith(' '):
            warning_message.append('\t' + line.rstrip())

        elif parsing and description and not line.lower().startswith(' '):
            parsing = False
            description = False

            # Check to see if the warning is in the list
            warning = [warning_file, warning_line, warning_message]
            if warning not in warning_list:
                # Add the warning to the list
                warning_list.append(warning)
                raw_warnings.append(translate_results.create_warning(warning_id, warning_file, warning_line,
                                                                     warning_message, ID_PREFIX, WARNING_LEVEL))

                # Increment the warning count
                warning_count = warning_count + 1

            else:
                logging.info('\t>> Duplicate warning omitted.')

    # Create the SCRUB output file
    translate_results.create_scrub_output_file(raw_warnings, parsed_output_file)


if __name__ == '__main__':
    parse_warnings(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]))



import os
import sys
import datetime
import shutil
import pathlib
from scrub.tools.parsers import translate_results


def parse_warnings(input_dir, output_format='legacy'):
    # Initialize variables
    source_dir = pathlib.Path(input_dir).parent
    output_dir = input_dir.joinpath('csv_output')
    timestamp = datetime.datetime.utcnow()

    # Make the output directory if it doesn't already exist
    if output_dir.exists():
        shutil.rmtree(str(output_dir))
    output_dir.mkdir()

    # Get a list of all the scrub files in the directory
    input_files = input_dir.glob('*.scrub')

    # Create the output file path
    output_file = output_dir.joinpath(timestamp.strftime("%m-%d-%Y") + '.csv')

    # Write the results out to the csv file
    with open(output_file, 'w') as output_fh:
        if output_format == 'generic':
            # Write out the header data
            output_fh.write('ID,Query,Description,Priority,File,Line\n')

            # Iterate through every file
            for input_file in input_files:

                # Parse the SCRUB files
                scrub_results = translate_results.parse_scrub(input_file, source_dir)

                # Iterate through every result
                for result in scrub_results:
                    output_fh.write('{},{},{},{},{},{}\n'.format(result.get('id'), result.get('query'),
                                                                 ' '.join(result.get('description')).replace(',', ''),
                                                                 result.get('priority'),
                                                                 os.path.relpath(str(result.get('file')), str(source_dir)),
                                                                 result.get('line')))

        else:
            # Iterate through every file
            for input_file in input_files:

                # Parse the SCRUB files
                scrub_results = translate_results.parse_scrub(input_file, source_dir)

                # Iterate through every result
                for result in scrub_results:
                    description_text = ' '.join(result.get('description')).replace(',', '').replace('\t', '').strip()
                    output_fh.write('{},{},{},{},{},{}:{},{},{},{},{}\n'.format(result.get('query'), description_text,
                                                                             result.get('priority'), description_text,
                                                                             os.path.relpath(str(result.get('file')), str(source_dir)),
                                                                             timestamp.strftime("%Y-%B-%d--%H-%M-%S"),
                                                                             os.path.relpath(str(result.get('file')), str(source_dir)),
                                                                             result.get('line'), result.get('line'), 0,
                                                                             0))


if __name__ == '__main__':
    if len(sys.argv) == 2:
        parse_warnings(pathlib.Path(sys.argv[1]).resolve())
    else:
        parse_warnings(pathlib.Path(sys.argv[1]).resolve(), sys.argv[2])



import re
import sys
import json
import pathlib
import logging
import traceback

WARNING_LINE_REGEX = r'^[a-z]+[0-9]+ <.*>.*:.*:.*:'


def create_warning(scrub_id, file, line, description, tool, priority='Low', query='', suppress=False):
    """This function creates an internal representation of a warning t be used for processing.

    Inputs:
        - id: Finding identifier of the format <tool><count> [string]
        - file: Absolute path to the source file referenced by the finding [string]
        - line: Line number of the source file being referenced by the findings [int]
        - description: Finding description [list of strings]
        - tool: Tool that generated the finding [string]
        - priority: Priority marking for the finding [Low/Med/High]
        - query: Tool query name that generated the finding [string]
        - suppress: Has this finding been suppressed? [bool]

    Outputs:
        - scrub_warning: Dictionary of warning data [dict]
    """

    # Create the warning
    scrub_warning = {'id': scrub_id,
                     'file': file,
                     'line': line,
                     'description': description,
                     'tool': tool,
                     'priority': priority,
                     'query': query,
                     'suppress': suppress}

    return scrub_warning


def format_scrub_warning(warning):
    """This function takes an internal representation of a warning and converts it into a SCRUB-formatted string.

    Inputs:
        - warning: Dictionary of finding data [dict]

    Outputs:
        - scrub_warning: SCRUB-formatted warning that can be written to the output file [string]
    """

    # Format the warning
    scrub_warning = (warning.get('id') + ' <' + warning.get('priority') + '> :' + str(warning.get('file')) + ':' +
                     str(warning.get('line')) + ': ' + warning.get('query') + '\n')

    # Add the description
    description = ''
    for line in warning.get('description'):
        description = description + '    ' + line + '\n'
    scrub_warning = scrub_warning + description + '\n'

    return scrub_warning


def create_scrub_output_file(warnings, output_file):
    """This function writes out raw warnings to a SCRUB formatted output file.

    Inputs:
        - warnings: Dictionary of raw warnings [list of dict]
        - output_file: Absolute path to output file to be created [string]
    """

    # Create the output file
    with open(output_file, 'w', encoding='utf-8') as output_fh:
        # Iterate through every raw warning
        for warning in warnings:
            # Check that the warning isn't suppressed
            if not warning['suppress']:
                # Create the SCRUB formatted warning
                scrub_warning = format_scrub_warning(warning)

                # Write the warning to the output file
                output_fh.write(scrub_warning)


def verify_sarif(sarif_data):
    """This function checks the SARIF data for known errors.

    Inputs:
        - sarif_data: A list of dictionaries containing analysis results [list of dict]

    Outputs:
        - valid_results: Boolean value that indicates if the data is error free [bool]
    """

    # Initialize variables
    execution_status = True

    # Check to make sure the analysis completed successfully
    if "invocations" in sarif_data:
        invocations_data = sarif_data['invocations'][0]

        if "executionSuccessful" in invocations_data:
            execution_status = invocations_data['executionSuccessful']
        elif "toolExecutionSuccessful" in invocations_data:
            execution_status = invocations_data['toolExecutionSuccessful']

    # Return an error if the SARIF file cannot be parsed
    if not execution_status:
        raise Exception


def get_rules_list(warnings):
    """This function gets a list of the rules contained in a set of warnings.

    Inputs:
        - warnings: List of warnings to be examined [list of dicts]

    Outputs:
        - rules_list: List of queries contained in the list of warnings [list of strings]
    """

    # Initialize variables
    rules_list = []

    # Iterate through every warning
    for warning in warnings:
        if warning['query'] not in rules_list and (warning['query'] != ''):
            rules_list.append(warning['query'])

    return rules_list


def run_sarif_merge(sarif_file1, sarif_file2, output_filename, sarif_version='sarifv2.1.0'):
    """This function merges two SARIF files into a single set of results.

    Inputs:
        - sarif_file1: Relative path to the first .sarif file to merge [string]
        - sarif_file2: Relative path to the second .sarif file to merge [string]
        - output_filename: Relative path to the merged .sarif output file [string]
        - sarif_version: Version to use for merged .sarif file, defaults to latest [string]

    Outputs:
        - exit_code: Exit code that represents whether the module completed with errors [int]
                       1 - reports any kind of failure with the merge (i.e. read/write, and schema mismatch)
                       2 - reports any unexpected errors that are not caught by exception handling
    """

    # Initialize variables
    custom_exit_code = 2

    try:
        with open(sarif_file1, 'r') as sarif_one, open(sarif_file2, 'r') as sarif_two:
            sarif_data1 = json.loads(sarif_one.read())
            sarif_data2 = json.loads(sarif_two.read())

        if sarif_data1 and sarif_data2:
            sarif_version = sarif_version.strip('sarifv')

            if (sarif_data1['version'] != sarif_version) or (sarif_data2['version'] != sarif_version):
                logging.warning('SARIF versions mismatched, converting files to: ' + sarif_version)

            merged_sarif = {
                '$schema': sarif_data1['$schema'],
                'version': sarif_version,
                'runs': [
                    sarif_data1['runs'][0],
                    sarif_data2['runs'][0]
                ]
            }

            with open(output_filename, 'w') as sarif_output:
                sarif_output.write(json.dumps(merged_sarif))

            # Update the exit code
            custom_exit_code = 0

    except:     # lgtm [py/catch-base-exception]
        logging.warning('Failed to merge these SARIF results, check traceback error for more details.')
        logging.warning(traceback.format_exc())
        custom_exit_code = 1

    finally:
        return custom_exit_code


def parse_scrub(scrub_file, source_root):
    """This function parses a scrub input file and returns a dictionary of findings.

    Inputs:
        - scrub_file: Absolute path to the SCRUB formatted file to bee parsed [string]
        - source_root: Root directory of source code [string]

    Outputs:
        - warning_dict: Dictionary of static analysis findings [dict]
        - query_list: List of queries found in the SCRUB input file [list of strings]
    """

    # Initialize variables
    warning_list = []

    # Import the data
    with open(scrub_file, 'r', encoding='utf-8') as input_fh:
        scrub_data = input_fh.read()

    # Split the warnings
    raw_warnings = list(filter(None, re.split('\n\n', scrub_data)))

    # Find all of the warnings in the file
    for raw_warning in raw_warnings:
        warning_lines = list(filter(None, re.split('\n', raw_warning.strip())))

        # Get the location information
        warning_info = list(filter(None, re.split(':', warning_lines[0].strip())))

        # Get the query name if it exists
        if len(warning_info) > 3:
            warning_query = list(warning_lines[0].split(': '))[-1].strip()
        else:
            warning_query = ''

        # Get the warning description
        warning_description = []
        for line in warning_lines[1:]:
            warning_description.append(line.rstrip())

        # Get the values of interest
        warning_id = warning_info[0].split()[0]
        warning_file = pathlib.Path(warning_info[1])
        warning_line = int(warning_info[2])
        warning_tool = re.sub(r'[0-9]', '', warning_id)
        warning_priority = re.sub('[<>]', '', warning_info[0].split()[-1])

        # Update the file path, if necessary
        if warning_file.anchor != '/':
            warning_file = source_root.joinpath(warning_file).resolve()

        # Add the warning to the dictionary
        warning_list.append(create_warning(warning_id, warning_file.resolve(), warning_line, warning_description, warning_tool,
                                           warning_priority, warning_query))

    return warning_list


def parse_sarif(sarif_filename, source_root, id_prefix=None):
    """This function parses all the SARIF results into the dictionary list of results.

    NOTE: SARIF schema is dependent on version number, current supported version is 2.1.0, but changes may need to be
          made for future updates.

    Inputs:
        - sarif_filename: Absolute path to the SARIF file to be parsed [string]
        - id_prefix: the tool name to assign to warnings, overrides name from actual results file. [string]

    Outputs:
        - results: List of the dictionary items that represent each filtered analysis result [list of dict]
        - rules: Dictionary of extended descriptions for all of the analysis queries [dict]
        - updated_source_dir: Updated source root directory based on contents of SARIF file [string]
    """

    # Initialize variables
    results = []
    warning_count = 1

    try:
        # Initialize variables
        updated_source_dir = source_root

        # Import the SARIF data
        with open(sarif_filename, 'r', encoding='utf-8') as sarif:
            sarif_data = json.loads(sarif.read())

        if sarif_data:
            # Create new dictionary item for each analysis result and append to results list
            schema_version = sarif_data['version']
            sarif_data = sarif_data.get('runs')[0]

            # Verify the results first
            verify_sarif(sarif_data)

            # Update the source root if it can be found in the SARIF data
            if "originalUriBaseIds" in sarif_data:
                for item in sarif_data.get("originalUriBaseIds").items():
                    updated_source_dir = pathlib.Path(item[-1]['uri'].replace('file://', '')).resolve()

            # TODO: replace key checks with booleans for all optional SARIF fields
            tool_name, analysis_rules, locations = '', [], []
            if schema_version == "2.1.0":
                tool_name = sarif_data.get('tool')['driver']['name'].split(' ')[0].lower()
                analysis_rules = {}
                # grab full descriptions for each analysis rule or query (only exists in current SARIF`` schema)
                if 'rules' in sarif_data['tool']['driver'].keys() and sarif_data['tool']['driver']['rules'] != []:
                    rules = sarif_data['tool']['driver']['rules']
                    for each in rules:
                        analysis_rules[each['id']] = each['fullDescription']['text']
                if 'artifacts' in sarif_data:
                    for file in sarif_data['artifacts']:
                        locations.append(file['location']['uri'])
            elif schema_version == "2.0.0":
                tool_name = sarif_data.get('tool')['name'].split(' ')[0].lower()
                # Get the list of file locations
                for file in sarif_data['files']:
                    if isinstance(file, dict):
                        locations.append(file['fileLocation']['uri'])  # TODO: duplicated code, modularize this task
                    else:
                        locations.append(file)
            if id_prefix:
                # prefer user defined tool name for scrub microfilter
                tool_name = id_prefix

            # Check to make sure there are results
            if 'results' in sarif_data.keys():
                # Iterate through the results
                for result in sarif_data['results']:
                    viewer_uri = ''
                    if 'suppressions' in result.keys() and result["suppressions"] != []:
                        suppress_warning = True
                    else:
                        suppress_warning = False

                    if result.get('hostedViewerUri') is not None:
                        viewer_uri = result['hostedViewerUri']

                    if 'ruleId' in result.keys():
                        warning_query = result['ruleId']

                    warning_description = [(result['message']['text'].replace('\n', ''))]
                    if 'codesonar' in tool_name.lower():
                        warning_description.append('Codesonar viewer: ' + viewer_uri)

                    location = result.get('locations')[0]['physicalLocation']  # this much is the same across versions
                    if schema_version == "2.1.0":
                        if 'rules' in sarif_data['tool']['driver'].keys() and sarif_data['tool']['driver']['rules'] != []:
                            if 'ruleId' in result.keys():
                                warning_description.append(analysis_rules[result['ruleId']].replace('\n', ''))

                        if 'uri' in location['artifactLocation'].keys() and location['artifactLocation']['uri'] != '':
                            warning_file = location['artifactLocation']['uri']
                        else:
                            file_index = result['locations'][0]['physicalLocation']['artifactLocation'].get('index')
                            warning_file = locations[file_index]
                        # TODO: use helper functions to do key checks? there are too many.
                    elif schema_version == "2.0.0":
                        if 'fileIndex' in location['fileLocation'].keys():
                            file_index = location['fileLocation']['fileIndex']  # mostly for CodeSonar compatibility
                            warning_file = locations[file_index]
                        else:
                            warning_file = location['fileLocation']['uri']

                    # Find the line number
                    if 'region' in location.keys():
                        warning_line = location['region']['startLine']
                    else:
                        warning_line = 0

                    # Set the ranking
                    if 'rank' in result.keys():
                        if int(result['rank']) > 56:
                            ranking = 'High'
                        elif 21 < int(result['rank']) <= 56:
                            ranking = 'Med'
                        else:
                            ranking = 'Low'
                    else:
                        ranking = 'Low'

                    # Fix the filepath
                    warning_file = pathlib.Path(warning_file.replace('file://', ''))
                    if warning_file.anchor != '/':
                        warning_file = updated_source_dir.joinpath(warning_file)

                    # Set the warning ID
                    warning_id = tool_name + str(warning_count).zfill(3)

                    # Add to the warning dictionary
                    results.append(create_warning(warning_id, warning_file.resolve(), warning_line, warning_description,
                                                  tool_name, ranking, warning_query, suppress_warning))

                    # Update the warning count
                    warning_count = warning_count + 1

        else:
            results = []

    except UnboundLocalError:
        logging.warning('Failed to parse file. SARIF schema version does not match SCRUB supported versions.')
        logging.warning(traceback.format_exc())

    except:      # lgtm [py/catch-base-exception]
        raise Exception

    return results


def create_sarif_output_file(results_list, sarif_version, output_file, source_root):
    """This function creates a SARIF formatted output file.

    Inputs:
        - results_list: List of dictionaries representing each warning [list of dicts]
        - sarif_version:
        - output_file:
        - source_root: Absolute path of source root directory [string]

    Returns:
        - output_file is created at the specified location
    """

    # Initialize variables
    result_item = {}
    rules_list = get_rules_list(results_list)
    sarif_output = {
        'version': sarif_version,
        '$schema': 'https://raw.githubusercontent.com/oasis-tcs/sarif-spec/master/Schemata/sarif-schema-2.1.0.json',
        'runs': [
                   {
                       'results': []
                   }
        ]
    }

    # Iterate through every warning
    for warning in results_list:
        file_index = 0

        # Set the priority level
        result_item['level'] = 'warning'

        # Set the file path
        if source_root in warning['file'].parents:
            warning_file = str(warning['file'].relative_to(source_root))
        else:
            warning_file = str(warning['file'])

        # Set the rule ID
        result_item['ruleId'] = warning['query']

        if warning.get('description') is not None:
            result_item['message'] = {
                'text': '\n'.join(warning['description'])
            }
        if sarif_version == '2.0.0':
            # TODO: CAREFUL USING 2.0.0, STRUCT IS NOT COMPLETELY DEPENDABLE, NEEDS WORK
            sarif_output['runs'][0].update({'resources': rules_list})

            sarif_output['runs'][0].update({'tool': results_list[0]['tool']})
            # TODO: get the logic right for deciding mime type... pass file_list like rules_list
            sarif_output['runs'][0]['files'] = [{
                "mimeType": "text/c",
                "fileLocation": {
                    "uri": results_list[0]['file']
                }
            }]
            result_item['locations'] = [
                {
                    'physicalLocation': {
                        'fileLocation': {
                            # TODO: get the logic right on properly indexing file paths (like with ruleIndex)
                            'fileIndex': file_index
                        },
                        'region': {
                            'startLine': warning['line']
                        }
                    }
                }
            ]
            file_index += 1
        elif sarif_version == '2.1.0':
            # Create the list of sarif rules
            sarif_rules = []
            for rule in rules_list:
                sarif_rules.append({
                    'id': rule,
                    'shortDescription': {
                        'text': rule
                    }
                })

            sarif_output['runs'][0]['tool'] = {
                'driver': {
                    'name': results_list[0]['tool'],
                    'rules': sarif_rules
                }
            }
            result_item['locations'] = [{
                'physicalLocation': {
                    'artifactLocation': {
                        'uri': warning_file,
                        'uriBaseId': str(source_root)
                    },
                    'region': {
                        'startLine': warning['line']
                    }
                }
            }]

        # append fixed warnings to results list and clean dict object
        sarif_output['runs'][0]['results'].append(result_item)
        result_item = {}

    # Create the output file
    with open(output_file, 'w') as output_fh:
        # output_fh.write('{}'.format(json.dumps(sarif_output, indent=4)))
        json.dump(sarif_output, output_fh, indent=4)


def perform_translation(input_file, output_file, source_root, output_format):
    """This function takes in an analysis results file in legacy format (.scrub), then parses and converts the contents
       of each analysis result into the SARIF format.

    Inputs:
        - scrub_filename: The name of the .sarif file to parse and convert. [string]
        - output_filename: The filename to output parsed results to. [string]

    Outputs:
        - custom_exit_code: Exit code that represents whether the module completed with errors.
                            1 - reports any kind of failure with the parser (i.e. read/write, and schema mismatch)
    """

    # Initialize the variables
    exit_code = 1
    parsed_results = []

    try:
        # Parse the input file
        if input_file.suffix == '.scrub':
            parsed_results = parse_scrub(input_file, source_root)

        elif input_file.suffix == '.sarif':
            parsed_results = parse_sarif(input_file, source_root)

        else:
            # TODO: This should generate an exception
            logging.error('Unknown input file type.')

        # Generate the desired output file
        if output_format == 'scrub':
            create_scrub_output_file(parsed_results, output_file)

        elif output_format.startswith('sarif'):
            # Determine the SARIF version
            sarif_version = output_format.strip('sarifv')

            # Generate the output file
            create_sarif_output_file(parsed_results, sarif_version, output_file, source_root)

        else:
            # TODO: This should generate an exception
            logging.error('Unknown output format type.')

        # Update the exit code
        exit_code = 0

    except:     # lgtm [py/catch-base-exception]
        logging.error('Translation could not be performed.')
        logging.error(traceback.format_exc())

        # Update the exit code
        exit_code = 100

    finally:
        return exit_code


if __name__ == '__main__':
    perform_translation(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]), pathlib.Path(sys.argv[3]), sys.argv[4])


import re
import os
import logging
import pwd
import traceback
import shutil
import subprocess
import pathlib
from scrub.utils import scrub_utilities
from scrub.utils.filtering import create_file_list


def login(bin_dir, login_flags):
    """This function logs in users to Collaborator.

    Inputs:
        - bin_dir: Absolute path to the Collaborator ccollab client [string]
        - login_flags: Command flags to be passed to the Collaborator command 'ccollab login' [string]
    """

    # Construct the call string
    if bin_dir == '':
        call_string = 'ccollab --quiet login ' + login_flags
    else:
        call_string = bin_dir + '/ccollab --quiet login ' + login_flags

    # Execute the command
    scrub_utilities.execute_command(call_string, os.environ.copy(), None, True)


def check_login(bin_dir):
    """This function checks to see if a user is logged in to the Collaborator server.

    Inputs:
        - bin_dir: Absolute path to the Collaborator ccollab client [string]
    """

    # Construct the call string
    if bin_dir == '':
        call_string = 'ccollab info'
    else:
        call_string = bin_dir + '/ccollab info'

    # Execute the command
    proc = subprocess.Popen(call_string, shell=True, env=os.environ.copy(), stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, encoding='utf-8')
    std_out, std_err = proc.communicate()

    # Check to see if a user is logged in
    if std_out.find('Connected as:') >= 0:
        authenticated = True
    else:
        authenticated = False

    return authenticated


def execute_ccollab(bin_dir, subcommand):
    """This function executes a given ccollab command.

    Inputs:
        - bin_dir: Absolute path to the Collaborator ccollab client [string]
        - subcommand: Command string and custom flags to be executed [string]
    """

    # Construct the call string
    if bin_dir == '':
        call_string = 'ccollab --no-browser --scm none ' + subcommand
    else:
        call_string = bin_dir + '/ccollab --no-browser --scm none ' + subcommand

    # Execute the command
    scrub_utilities.execute_command(call_string, os.environ.copy())


def get_review_id(log_file):
    """This function finds the review ID from the Collaborator log file.

    Inputs:
        - log_file: Absolute path to the Collaborator log file [string]

    Outputs:
        review_id: Review ID number found in the Collaborator log file [string]
    """

    # Initialize variables
    review_id = 0

    # Import the log data
    with open(log_file, 'r') as input_fh:
        review_id_data = input_fh.readlines()

    # Find the review ID
    for line in review_id_data:
        if 'Successfully created review' in line:
            review_id = re.split(' ', line.strip())[-1].replace('.', '')

    return review_id


def get_defects(scrub_file):
    """This function parses the input file and creates an internal dictionary of findings.

    Inputs:
        - scrub_file: Absolute path to the SCRUB-formatted results file [string]

    Outputs:
        defect_list: List of defects found within the results file [list of dict]
    """

    # Initialize variables
    special_chars_dict = {'"': "&quot;",
                          "'": "&apos;",
                          "<": "&lt;",
                          ">": "&gt;",
                          "&": "&amp;"}
    defect_list = []

    # Import the defects from the file of interest
    with open(scrub_file, 'r', encoding='UTF-8') as fh:
        results = fh.read()

    # Split the results into defects
    defects = results.split('\n\n')

    # Extract defect info
    for raw_defect in defects:
        if raw_defect:
            # Initialize the dict
            defect = {'id': None,
                      'file': None,
                      'line': None,
                      'priority': None,
                      'description': None,
                      'tool': None}

            # Replace any whitespace with single space
            raw_defect = re.sub(r'\s+', ' ', raw_defect)

            # Get tool abbrv, defect num, severity, file name, line num, description
            raw_defect = re.match(r'(\D+\d*\D+)(\d*) <(\D+)> :(\S+):(\d+): (.+)', raw_defect)

            if 'p10' in scrub_file.stem:
                defect.update({'tool': 'p10_' + raw_defect.group(1)})
            else:
                defect.update({'tool': raw_defect.group(1)})
            defect_num = raw_defect.group(2)

            defect.update({'id': defect.get('tool') + defect_num})
            defect.update({'priority': raw_defect.group(3)})
            defect.update({'file': raw_defect.group(4)})
            defect.update({'line': raw_defect.group(5)})
            defect.update({'description': raw_defect.group(6)})

            # Convert special chars in description
            for key in special_chars_dict.keys():
                defect.update({'description': defect.get('description').replace(key, special_chars_dict[key])})

            # Add the defect to the list
            defect_list.append(defect)

    return defect_list


def construct_xml_global_options(url, username):
    """This function constructs the global options portion of the XML file.

    Inputs:
        - url: URL of the Collaborator server [string]
        - username: Username for the Collaborator user [string]

    Outputs:
        - global_options_string: A string containing the global options [string]
    """

    # Create the global options string
    global_options_string = ('    <global-options>\n' +
                             '        <no-browser/>\n' +
                             '        <scm>none</scm>\n' +
                             '        <url>{}</url>\n'.format(url) +
                             '        <user>{}</user>\n'.format(username) +
                             '    </global-options>\n')

    return global_options_string


def create_batch_xml_file_upload(file_list, review_id, source_root):
    """This function creates the XML file that is used to upload review files to Collaborator.

    Inputs:
        - file_list: List of files to be uploaded to Collaborator [list of strings]
        - review_id: ID of review to upload files [string]
        - source_root: Absolute path to the source code root directory [string]
    """

    # Initialize the variables
    file_upload_xml_batch = ('    <addfiles>\n' +
                             '        <review>{}</review>\n'.format(review_id) +
                             '        <relative-to>{}</relative-to>\n'.format(source_root) +
                             '        <upload-comment>Initial SCRUB upload</upload-comment>\n')

    # Iterate through every file and add it to the upload
    for file in file_list:
        file_upload_xml_batch = file_upload_xml_batch + '        <file-path>{}</file-path>\n'.format(file.strip())

    file_upload_xml_batch = file_upload_xml_batch + '    </addfiles>\n'

    return file_upload_xml_batch


def create_batch_xml_comment_upload(file_list, comment_list, review_id):
    """This function creates the XML snippet that is used to upload review comments to Collaborator.

    Inputs:
        - file_list: List of files that are present within the review [list of string]
        - defect_list: List of defects to be uploaded to the Collaborator review [list of dicts]
        - review_id: ID of review to upload files [string]
    """

    # Initialize variables
    comment_xml_list = ''

    # Write out every defect for files of interest
    for comment in comment_list:
        if comment.get('file') in file_list:
            # Construct the comment xml
            comment_xml = ('    <admin_review_comment_create>\n' +
                           '        <review>{}</review>\n'.format(review_id) +
                           '        <file>{}</file>\n'.format(comment.get('file')) +
                           '        <line-number>{}</line-number>\n'.format(comment.get('line')) +
                           '        <comment>{}: {}</comment>\n'.format(comment.get('tool'), comment.get('description')) +
                           '    </admin_review_comment_create>\n')

            # Add it to the list
            comment_xml_list = comment_xml_list + comment_xml

    return comment_xml_list


def create_batch_xml_defect_upload(file_list, defect_list, review_id):
    """This function creates the XML snippet that is used to upload review defects to Collaborator.

    Inputs:
        - file_list: List of files that are present within the review [list of string]
        - defect_list: List of defects to be uploaded to the Collaborator review [list of dicts]
        - review_id: ID of review to upload files [string]
    """

    # Initialize variables
    defect_xml_list = ''

    # Write out every defect for files of interest
    for defect in defect_list:
        if defect.get('file') in file_list:
            # Parse the priority level
            if defect.get('priority') == 'High':
                severity = 'Major'
            elif defect.get('priority') == 'Med':
                severity = 'Moderate'
            else:
                severity = 'Minor'

            # Create the defect XML
            defect_xml = ('    <admin_review_defect_create>\n'+
                                 '        <custom-field>Severity={}</custom-field>\n'.format(severity) +
                                 '        <review>{}</review>\n'.format(review_id) +
                                 '        <file>{}</file>\n'.format(defect.get('file')) +
                                 '        <line-number>{}</line-number>\n'.format(defect.get('line')) +
                                 '        <comment>{}: {}</comment>\n'.format(defect.get('tool'),
                                                                              defect.get('description')) +
                                 '    </admin_review_defect_create>\n')

            # Update the list of defects
            defect_xml_list = defect_xml_list + defect_xml

    return defect_xml_list


def initialize_upload(tool_conf_data):
    """This function prepares to upload review data to Collaborator.

    Inputs:
        - tool_conf_data: Dictionary of scrub.cfg input variables [dict]
    """

    # Create the filtering files
    create_filtering_files(tool_conf_data)

    # Check to see if a user is logged in
    if not check_login(tool_conf_data.get('collaborator_ccollab_location')):
        # Prompt user to log in
        login_flags = tool_conf_data.get('collaborator_server') + ' ' + tool_conf_data.get('collaborator_username')
        login(tool_conf_data.get('collaborator_ccollab_location'), login_flags)

    # Create the review
    subcommand = ('admin review create --creator ' + tool_conf_data.get('collaborator_username') +
                  ' --restrict-access \"' + tool_conf_data.get('collaborator_review_access') + '\" --template \"' +
                  tool_conf_data.get('collaborator_review_template') + '\" --title \"' +
                  tool_conf_data.get('collaborator_review_title') + '\"')
    if tool_conf_data.get('collaborator_review_group'):
        subcommand = (subcommand + ' --group \"' + tool_conf_data.get('collaborator_review_group') + '\"')
    execute_ccollab(tool_conf_data.get('collaborator_ccollab_location'), subcommand)

    # Get the review ID and add it to the configuration data
    review_id = get_review_id(tool_conf_data.get('collaborator_log_file'))
    tool_conf_data.update({'collaborator_review_id': int(review_id)})

    # Set the review xml file path
    batch_command_file = tool_conf_data.get('collaborator_upload_dir').joinpath('collaborator_review_' +
                                                                                str(review_id) + '_batch.xml')
    tool_conf_data.update({'batch_command_file': batch_command_file})
    # tool_conf_data.update({'collaborator_review_xml_comments': comments_xml})

    # Set the author of the review
    subcommand = ('admin review set-participants ' + review_id + ' --participant author=' +
                  tool_conf_data.get('collaborator_username'))
    execute_ccollab(tool_conf_data.get('collaborator_ccollab_location'), subcommand)


def perform_upload(tool_conf_data):
    """This function uploads review data to the Collaborator review.

    Inputs:
        - tool_conf_data: Dictionary of scrub.cfg input variables [dict]
    """

    # Initialize variables
    tool_defect_types = []
    defect_list = []

    # Get a list of the source code files to be uploaded
    with open(tool_conf_data.get('collaborator_filtering_output_file'), 'r') as fh:
        file_list = list(filter(None, re.split('\n', fh.read())))

    # Check if any files specified
    if tool_conf_data.get('collaborator_src_files'):
        for results_file in tool_conf_data.get('collaborator_src_files').split(','):
            tool_name = results_file[0:-6]
            tool_defect_types.append(tool_name)
    else:
        for filename in tool_conf_data.get('scrub_analysis_dir').glob('*.scrub'):
            if 'raw' not in filename.stem:
                tool_name = filename.stem
                tool_defect_types.append(tool_name)

    # Get the defects
    for tool_name in tool_defect_types:
        scrub_file = tool_conf_data.get('scrub_analysis_dir').joinpath(tool_name + '.scrub')

        # Add the defects to the review
        defect_list = defect_list + get_defects(scrub_file)

    # Create XML data for uploading files
    file_xml_data = create_batch_xml_file_upload(file_list, tool_conf_data.get('collaborator_review_id'),
                                                 tool_conf_data.get('source_dir'))

    if tool_conf_data.get('collaborator_finding_level').lower() == 'defect':
        finding_xml_data = create_batch_xml_defect_upload(file_list, defect_list,
                                                          tool_conf_data.get('collaborator_review_id'))
    else:
        finding_xml_data = create_batch_xml_comment_upload(file_list, defect_list,
                                                           tool_conf_data.get('collaborator_review_id'))

    # Add a generic comment if the finding upload list is empty
    if finding_xml_data == '':
        finding_xml_data = ('    <admin_review_comment_create>\n' +
                            '        <review>{}</review>\n'.format(tool_conf_data.get('collaborator_review_id')) +
                            '        <comment>No findings were found to include in this review.</comment>\n' +
                            '    </admin_review_comment_create>\n')

    # Create the XML batch file
    with open(tool_conf_data.get('batch_command_file'), 'w+') as output_fh:
        output_fh.write('<batch-commands>\n')
        output_fh.write('{}'.format(construct_xml_global_options(tool_conf_data.get('collaborator_server'),
                                                                 tool_conf_data.get('collaborator_username'))))
        output_fh.write('{}'.format(file_xml_data))
        output_fh.write('{}'.format(finding_xml_data))
        output_fh.write('</batch-commands>')

    # Perform the upload
    subcommand = ('admin batch ' + str(tool_conf_data.get('batch_command_file')))
    execute_ccollab(tool_conf_data.get('collaborator_ccollab_location'), subcommand)


def create_filtering_files(tool_conf_data):
    """This function creates the filtering files needed for Collaborator upload.

    Inputs:
        - tool_conf_data: Dictionary of scrub.cfg input variables [dict]
    """

    # Print a status message
    logging.info('\t>> Checking analysis filtering output file.')

    # Determine if the baseline filtering list exists, or should be created
    if not tool_conf_data.get('filtering_output_file').exists():
        # Print a status message
        logging.info('\t>> No analysis filtering output was found. Creating output now.')

        # Create the output file
        create_file_list.create_file_list(tool_conf_data.get('source_dir'),
                                          tool_conf_data.get('filtering_output_file'),
                                          tool_conf_data.get('analysis_filters'))

    # Create the file list based on filtering options
    create_file_list.create_file_list(tool_conf_data.get('source_dir'),
                                      tool_conf_data.get('collaborator_filtering_output_file'),
                                      tool_conf_data.get('collaborator_filters'),
                                      tool_conf_data.get('filtering_output_file'))


def run_analysis(baseline_conf_data, console_logging=logging.INFO, override=False):
    """This function runs this module based on input from the scrub.cfg file

    Inputs:
        - baseline_conf_data: Dictionary of raw scrub.cfg configuration parameters [dict]
        - console_logging: Level of console logging information to print to console [optional] [enum]
        - override: Force tool execution? [optional] [bool]

    Outputs:
        - collaborator.log: Log file for the Collaborator upload, where uid is the user ID
        - review.xml: XML file containing appropriate Collaborator commands to upload source code and SCRUB results
        - stdout and stderr data will be directed to log_file
    """

    # Import the config file data
    tool_conf_data = baseline_conf_data.copy()
    initialize_analysis(tool_conf_data)

    # Initialize variables
    collaborator_exit_code = 2
    attempt_analysis = tool_conf_data.get('collaborator_export') or override

    # Determine if Collaborator upload can be run
    if attempt_analysis:
        try:
            # Create the logging file, if it doesn't already exist
            scrub_utilities.create_logger(tool_conf_data.get('collaborator_log_file'), console_logging)

            # Print a status message
            logging.info('')
            logging.info('Perform Collaborator export...')

            # Prepare for the upload
            initialize_upload(tool_conf_data)

            # Perform the upload
            perform_upload(tool_conf_data)

            # Set the exit code
            collaborator_exit_code = 0

        except scrub_utilities.CommandExecutionError:
            # Print a warning message
            logging.warning('Collaborator export could not be performed. Please see log file %s for more information.',
                            str(tool_conf_data.get('collaborator_log_file')))

            # Print the exception traceback
            logging.warning(traceback.format_exc())

            # Set the exit code
            collaborator_exit_code = 1

        except:     # lgtm [py/catch-base-exception]
            # Print a warning message
            logging.warning('Collaborator export could not be performed. Please see log file %s for more information.',
                            str(tool_conf_data.get('collaborator_log_file')))

            # Print the exception traceback
            logging.warning(traceback.format_exc())

            # Set the exit code
            collaborator_exit_code = 100

        finally:
            # Delete the review if necessary
            if (collaborator_exit_code > 0) and (tool_conf_data.get('collaborator_review_id') > 0):
                subcommand = ('admin review delete ' + str(tool_conf_data.get('collaborator_review_id')))
                execute_ccollab(str(tool_conf_data.get('collaborator_ccollab_location')), subcommand)

            # Close the loggers
            logging.getLogger().handlers = []

            # Move the log file to line up with the review id, if it exists
            if tool_conf_data.get('collaborator_log_file').exists() and tool_conf_data.get('collaborator_review_id') > 0:
                shutil.move(tool_conf_data.get('collaborator_log_file'),
                            tool_conf_data.get('scrub_log_dir').joinpath('collaborator_' + str(tool_conf_data.get('collaborator_review_id')) + '.log'))

    # Return the exit code
    return collaborator_exit_code


def initialize_analysis(tool_conf_data):
    """The purpose of this function is to prepare the tool to perform analysis.

    Inputs:
        - tool_conf_data: Dictionary of scrub.cfg input variables [dict]
    """

    # Initialize the derived variables
    current_user = pwd.getpwuid(os.getuid()).pw_name
    collaborator_upload_dir = tool_conf_data.get('scrub_analysis_dir').joinpath('collaborator_upload')
    collaborator_log_file = tool_conf_data.get('scrub_log_dir').joinpath('collaborator_' + current_user + '.log')
    collaborator_filtering_output_file = collaborator_upload_dir.joinpath('SCRUBCollaboratorFilteringList')

    # Add derived values to the dictionary
    tool_conf_data.update({'collaborator_log_file': collaborator_log_file})
    tool_conf_data.update({'collaborator_upload_dir': collaborator_upload_dir})
    tool_conf_data.update({'collaborator_filtering_output_file': collaborator_filtering_output_file})
    tool_conf_data.update({'collaborator_review_id': 0})

    # Set the username if necessary
    if tool_conf_data.get('collaborator_username') == '':
        tool_conf_data.update({'collaborator_username': current_user})

    # Create the working directory if it doesn't already exist
    if not collaborator_upload_dir.exists():
        collaborator_upload_dir.mkdir()

    # Resolve the COLLABORATOR_FILTERS variable
    tool_conf_data.update({'collaborator_filters': pathlib.Path(tool_conf_data.get('collaborator_filters'))})

    # Determine if Collaborator can be run
    if not (tool_conf_data.get('collaborator_server')):
        # Update the analysis flag if necessary
        if tool_conf_data.get('collaborator_export'):
            tool_conf_data.update({'collaborator_export': False})

            # Print a status message
            print('\nWARNING: Unable to perform Collaborator export. Required configuration inputs are missing.')
            print('\tRequired inputs: COLLABORATOR_SERVER\n')



import sys
import json
import pathlib
from scrub.tools.parsers import translate_results

ID_PREFIX = 'sonarqube'


def parse_warnings(results_dir, parsed_output_file, source_root, sonarqube_url):
    """This function parses the raw SonarQube warnings into the SCRUB format.
        Inputs:
            - results_dir: Absolute path to the raw SonarQube output file directory [string]
            - parsed_output_file: Absolute path to the file where the parsed warnings will be stored [string]
            - source_root: Absolute path to the source root directory [string]
            - sonarqube_url: URL of the SonarQube server [string]
        """

    # Initialize the variables
    warning_count = 1
    raw_warnings = []

    # Find all the raw findings results files in the directory
    findings_results_files = results_dir.glob('*.json')

    # Iterate through every issues results file
    for raw_findings_file in findings_results_files:
        # Read in the input data
        with open(raw_findings_file, 'r') as input_fh:
            input_data = json.loads(input_fh.read())

        # Iterate through every finding in the input file
        if 'issues' in input_data.keys():
            findings = input_data['issues']
        elif 'hotspots' in input_data.keys():
            findings = input_data['hotspots']
        else:
            break

        for finding in findings:
            # Check to see if the warning should be suppressed
            if 'resolution' in finding.keys():
                suppression = True
            else:
                suppression = False

            # Parse the finding
            warning_file = source_root.joinpath(finding['component'].split(':')[-1]).resolve()
            warning_message = finding['message'].splitlines()
            warning_id = ID_PREFIX + str(warning_count).zfill(3)

            # Get a link to the finding
            if 'sonarqube_hotspots' in raw_findings_file.stem:
                warning_link = sonarqube_url + '/security_hotspots?id=' + finding['project'] + '&hotspots=' + finding[
                    'key']
            else:
                warning_link = sonarqube_url + '/project/issues?id=' + finding['project'] + '&open=' + finding['key']

            # Add the link to the warning message
            warning_message.append(warning_link)

            # Parse the query if it exists
            if 'rule' in finding.keys():
                warning_query = finding['rule']
            elif 'ruleKey' in finding.keys():
                warning_query = finding['ruleKey']
            else:
                warning_query = ''

            # Get the priority value from SonarQube
            if 'vulnerabilityProbability' in finding.keys():
                sonarqube_priority = finding['vulnerabilityProbability'].lower()
            elif 'severity' in finding.keys():
                sonarqube_priority = finding['severity'].lower()
            else:
                sonarqube_priority = 'low'

            # Translate the priority to High/Med/Low
            if (sonarqube_priority == 'blocker') or (sonarqube_priority == 'critical') or (sonarqube_priority == 'high'):
                priority = 'High'
            elif (sonarqube_priority == 'major') or (sonarqube_priority == 'medium'):
                priority = 'Med'
            elif (sonarqube_priority == 'minor') or (sonarqube_priority == 'info') or (sonarqube_priority == 'low'):
                priority = 'Low'
            else:
                priority = 'Low'

            # Get the line number
            if 'line' in finding.keys():
                warning_line = int(finding['line'])
            elif 'textRange' in finding.keys():
                warning_line = int(finding['textRange']['startLine'])
            else:
                warning_line = 0

            # Add to the warning dictionary
            raw_warnings.append(translate_results.create_warning(warning_id, warning_file, warning_line,
                                                                 warning_message, ID_PREFIX, priority,
                                                                 warning_query, suppression))

            # Increment the warning count
            warning_count = warning_count + 1

    # Create the SCRUB output file
    translate_results.create_scrub_output_file(raw_warnings, parsed_output_file)


if __name__ == '__main__':
    parse_warnings(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]), pathlib.Path(sys.argv[3]), sys.argv[4])



import re
import sys
import pathlib
import logging
from scrub.tools.parsers import translate_results

WARNING_LEVEL = 'Low'
ID_PREFIX = 'javac'


def parse_warnings(raw_input_file, parsed_output_file):
    """This function parses the raw javac compiler warnings into the SCRUB format.

    Inputs:
        - raw_input_file: Absolute path to the raw javac compiler log containing warnings [string]
        - parsed_output_file: Absolute path to the file where the parsed warnings will be stored [string]
    """

    # Initialize variables
    warning_count = 1

    # Print a status message
    logging.info('')
    logging.info('\tParsing results...')
    logging.info('\t>> Executing command: get_javac_warnings.parse_warnings(%s, %s)', str(raw_input_file),
                 str(parsed_output_file))
    logging.info('\t>> From directory: %s', str(pathlib.Path().absolute()))

    # Import the data from the input file
    with open(raw_input_file, 'r') as input_fh:
        input_data = input_fh.readlines()

    # Iterate through every line of the input file
    raw_warnings = []
    for line in input_data:
        # Check to see if there is a warning or error
        if (' warning: ' in line) or (' error: ' in line):
            # Split the line and store the file name and line
            line_split = list(filter(None, re.split('[ :]', line.strip())))
            warning_file = pathlib.Path(line_split[0]).resolve()
            warning_line = line_split[1]

            # Split the line and store the message and type of warning
            line_split = list(filter(None, re.split(':', line.strip())))
            warning_message = ['Javac Compiler Warning: ' + line_split[-1].strip()]
            warning_type = line_split[-2].strip()
            warning_id = ID_PREFIX + str(warning_count).zfill(3)

            # Add to the warning dictionary
            raw_warnings.append(translate_results.create_warning(warning_id, warning_file, warning_line,
                                                                 warning_message, ID_PREFIX, WARNING_LEVEL,
                                                                 warning_type))

            # Increment the counter
            warning_count = warning_count + 1

    # Create the output file
    translate_results.create_scrub_output_file(raw_warnings, parsed_output_file)


if __name__ == '__main__':
    parse_warnings(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]))



import sys
import xml.etree.ElementTree
import pathlib
import logging


def parse_warnings(input_file, output_file, codesonar_hub, exclude_p10=False):
    """This function parses the raw CodeSonar warnings into the SCRUB format.

    Inputs:
        - input_file: Full path to the file containing raw CodeSonar warnings [string]
        - output_file: Full path to the file where the parsed warnings will be stored [string]
        - codesonar_hub: Location of the CodeSonar Hub [string]

    Outputs:
        - output_file: All parsed warnings will be written to the output_file
    """

    # Print a status message
    logging.info('\t>> Executing command: get_codesonar_warnings.parse_warnings(%s, %s)', input_file, output_file)
    logging.info('\t>> From directory: %s', str(pathlib.Path().absolute()))

    # Initialize the variables
    warning_count = 1
    id_prefix = 'codesonar'
    p10_only = False
    filter_list = ["Goto Statement",  # P10: Rule 1
                   "Recursion",  # P10: Rule 1
                   "Use of < setjmp.h >",  # P10: Rule 1
                   "Use of longjmp",  # P10: Rule 1
                   "Use of setjmp",  # P10: Rule 1
                   "Potential Unbounded Loop",  # P10: Rule 2
                   "Dynamic Allocation After Initialization",  # P10: Rule 3
                   "Function Too Long",  # P10: Rule 4
                   "Not Enough Assertions",  # P10: Rule 5
                   "Scope Could Be File Static",  # P10: Rule 6
                   "Scope Could Be Local Static",  # P10: Rule 6
                   "Ignored Return Value",  # P10: Rule 7
                   "Unchecked Parameter Dereference",  # P10: Rule 7
                   "## Follows # Operator",  # P10: Rule 8
                   "Conditional Compilation",  # P10: Rule 8
                   "Macro Does Not End With } or )",  # P10: Rule 8
                   "Macro Does Not Start With { or (",  # P10: Rule 8
                   "Macro Name is C Keyword",  # P10: Rule 8
                   "Macro Uses # Operator",  # P10: Rule 8
                   "Macro Uses ## Operator",  # P10: Rule 8
                   "Non-Boolean Preprocessor Expression",  # P10: Rule 8
                   "Preprocessing Directives in Macro Argument",  # P10: Rule 8
                   "Recursive Macro",  # P10: Rule 8
                   "Unbalanced Parenthesis",  # P10: Rule 8
                   "Use of <stdio.h> Input/Output Macro",  # P10: Rule 8
                   "Use of <wchar.h> Input/Output Macro",  # P10: Rule 8
                   "Variadic Macro",  # P10: Rule 8
                   "Function Pointer",  # P10: Rule 9
                   "Macro Uses [] Operator",  # P10: Rule 9
                   "Macro Uses -> Operator",  # P10: Rule 9
                   "Macro Uses Unary * Operator",  # P10: Rule 9
                   "Pointer Type Inside Typedef",  # P10: Rule 9
                   "Too Many Dereferences",  # P10: Rule 9
                   "Not All Warnings Are Enabled",  # P10: Rule 10
                   "Warnings Not Treated As Errors"]  # P10: Rule 10

    # Determine if the output is P10 results
    if 'p10' in output_file.stem:
        p10_only = True

    # Import the input data
    codesonar_data = xml.etree.ElementTree.parse(input_file).getroot()

    # Create the output file
    with open(output_file, 'w+') as output_fh:
        # Find all the tables in the XML file
        for warning in codesonar_data.findall('warning'):
            # Parse the warning into SCRUB format
            warning_instance_id = warning.get("url").split('/')[-1].split('.')[0]
            warning_file = warning.findall("file_path")[0].text
            warning_line = int(warning.findall("line_number")[0].text)
            warning_class = warning.findall("class")[0].text
            if str(warning.findall("procedure")[0].text).lower() == 'none':
                warning_procedure = 'undefined procedure'
            else:
                warning_procedure = warning.findall("procedure")[0].text
            warning_summary = warning_class + ' found in ' + warning_procedure
            warning_link = codesonar_hub + '/warninginstance/' + warning_instance_id + '.html'

            # Get the ranking information
            if int(warning.findall("score")[0].text) > 56:
                warning_level = 'High'
            elif 21 < int(warning.findall("score")[0].text) <= 56:
                warning_level = 'Med'
            else:
                warning_level = 'Low'

            # Check to see if the warnings should be filtered for P10 results
            if p10_only:
                # Check to see if the warning class is associated with P10 warnings
                if warning_class.strip() in filter_list:
                    # Write the warnings to the output file
                    output_fh.write('%s%03d <%s> :%s:%d: %s\n\tCodeSonar P10 Warning: %s\n\t%s\n\n' %
                                    (id_prefix, warning_count, warning_level, warning_file, warning_line,
                                     warning_class, warning_summary, warning_link))

                    # Increase the warning_count
                    warning_count = warning_count + 1

            elif exclude_p10:
                # Check to see if the warning class is associated with P10 warnings
                if warning_class.strip() not in filter_list:
                    # Write the warnings to the output file
                    output_fh.write('%s%03d <%s> :%s:%d: %s\n\t%s\n\t%s\n\n' %
                                    (id_prefix, warning_count, warning_level, warning_file, warning_line,
                                     warning_class, warning_summary, warning_link))

                    # Increase the warning_count
                    warning_count = warning_count + 1

            else:
                # Write the warnings to the output file
                output_fh.write('%s%03d <%s> :%s:%d: %s\n\t%s\n\t%s\n\n' %
                                (id_prefix, warning_count, warning_level, warning_file, warning_line,
                                 warning_class, warning_summary, warning_link))

                # Increase the warning_count
                warning_count = warning_count + 1


if __name__ == '__main__':
    parse_warnings(pathlib.Path(sys.argv[1]), pathlib.Path(sys.argv[2]), sys.argv[3])
